@@ -1,27 +1,34 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
+# cython: language_level=3
 
 import re
 import sympy
 from functools import total_ordering
 import importlib
 from itertools import chain
-
 import typing
 from typing import Any, cast
 
+from mathics.version import __version__  # noqa used in loading to check consistency.
+
+from mathics.core.convert import from_sympy
 from mathics.core.definitions import Definition
 from mathics.core.parser.util import SystemDefinitions, PyMathicsDefinitions
 from mathics.core.rules import Rule, BuiltinRule, Pattern
 from mathics.core.expression import (
     BaseExpression,
     Expression,
-    Symbol,
-    String,
     Integer,
+    MachineReal,
+    PrecisionReal,
+    String,
+    Symbol,
+    SymbolTrue,
+    SymbolFalse,
     ensure_context,
     strip_context,
 )
+from mathics.core.numbers import get_precision, PrecisionValueError
 
 
 def get_option(options, name, evaluation, pop=False, evaluate=True):
@@ -30,20 +37,17 @@ def get_option(options, name, evaluation, pop=False, evaluate=True):
     # matter. Also, the quoted string form "X" is ok. all these
     # variants name the same option. this matches Wolfram Language
     # behaviour.
-
+    name = strip_context(name)
     contexts = (s + "%s" for s in evaluation.definitions.get_context_path())
 
     for variant in chain(contexts, ('"%s"',)):
         resolved_name = variant % name
-
         if pop:
             value = options.pop(resolved_name, None)
         else:
             value = options.get(resolved_name)
-
         if value is not None:
             return value.evaluate(evaluation) if evaluate else value
-
     return None
 
 
@@ -53,9 +57,10 @@ def has_option(options, name, evaluation):
 
 mathics_to_python = {}
 
+
 class Builtin(object):
     name: typing.Optional[str] = None
-    context = "System`"
+    context = ""
     abstract = False
     attributes: typing.Tuple[Any, ...] = ()
     rules: typing.Dict[str, Any] = {}
@@ -68,7 +73,7 @@ class Builtin(object):
         if kwargs.get("expression", None) is not False:
             return Expression(cls.get_name(), *args)
         else:
-            instance = super(Builtin, cls).__new__(cls)
+            instance = super().__new__(cls)
             if not instance.formats:
                 # Reset formats so that not every instance shares the same
                 # empty dict {}
@@ -76,20 +81,21 @@ class Builtin(object):
             return instance
 
     def __init__(self, *args, **kwargs):
-        super(Builtin, self).__init__()
+        super().__init__()
         if hasattr(self, "python_equivalent"):
             mathics_to_python[self.get_name()] = self.python_equivalent
 
     def contribute(self, definitions, is_pymodule=False):
         from mathics.core.parser import parse_builtin_rule
 
-        if is_pymodule:
-            name = "PyMathics`" + self.get_name(short=True)
-        else:
-            name = self.get_name()
+        # Set the default context
+        if not self.context:
+            self.context = "Pymathics`" if is_pymodule else "System`"
 
+        name = self.get_name()
         options = {}
         option_syntax = "Warn"
+
         for option, value in self.options.items():
             if option == "$OptionSyntax":
                 option_syntax = value
@@ -145,9 +151,7 @@ class Builtin(object):
 
         for pattern, function in self.get_functions(is_pymodule=is_pymodule):
             rules.append(
-                BuiltinRule(
-                    name, pattern, function, check_options, system=not is_pymodule
-                )
+                BuiltinRule(name, pattern, function, check_options, system=True)
             )
         for pattern, replace in self.rules.items():
             if not isinstance(pattern, BaseExpression):
@@ -259,6 +263,7 @@ class Builtin(object):
                 pattern = Expression("Default", Symbol(name), Integer(spec))
             if pattern is not None:
                 defaults.append(Rule(pattern, value, system=True))
+
         definition = Definition(
             name=name,
             rules=rules,
@@ -267,6 +272,7 @@ class Builtin(object):
             attributes=attributes,
             options=options,
             defaultvalues=defaults,
+            builtin=self,
         )
         if is_pymodule:
             definitions.pymathics[name] = definition
@@ -311,15 +317,14 @@ class Builtin(object):
                     pattern = m.group(2)
                 else:
                     attrs = []
-                if is_pymodule:
-                    name = "PyMathics`" + self.get_name(short=True)
-                else:
-                    name = self.get_name()
-
+                # if is_pymodule:
+                #    name = ensure_context(self.get_name(short=True), "Pymathics")
+                # else:
+                name = self.get_name()
                 pattern = pattern % {"name": name}
                 definition_class = (
                     PyMathicsDefinitions() if is_pymodule else SystemDefinitions()
-                    )
+                )
                 pattern = parse_builtin_rule(pattern, definition_class)
                 if unavailable_function:
                     function = unavailable_function
@@ -363,11 +368,11 @@ class Builtin(object):
         return None, s
 
 
-class InstancableBuiltin(Builtin):
+class InstanceableBuiltin(Builtin):
     def __new__(cls, *args, **kwargs):
         new_kwargs = kwargs.copy()
         new_kwargs["expression"] = False
-        instance = super(InstancableBuiltin, cls).__new__(cls, *args, **new_kwargs)
+        instance = super().__new__(cls, *args, **new_kwargs)
         if not instance.formats:
             # Reset formats so that not every instance shares the same empty
             # dict {}
@@ -376,7 +381,7 @@ class InstancableBuiltin(Builtin):
             try:
                 instance.init(*args, **kwargs)
             except TypeError:
-                # TypeError occurs when unpickling instance, e.g. PatterObject,
+                # TypeError occurs when unpickling instance, e.g. PatternObject,
                 # because parameter expr is not given. This should no be a
                 # problem, as pickled objects need their init-method not
                 # being called.
@@ -393,7 +398,7 @@ class AtomBuiltin(Builtin):
     # see Image[] for an example of this.
 
     def get_name(self, short=False) -> str:
-        name = super(AtomBuiltin, self).get_name(short=short)
+        name = super().get_name(short=short)
         return re.sub(r"Atom$", "", name)
 
 
@@ -417,15 +422,35 @@ class Operator(Builtin):
 
 class Predefined(Builtin):
     def get_functions(self, prefix="apply", is_pymodule=False):
-        functions = list(super(Predefined, self).get_functions(prefix))
+        functions = list(super().get_functions(prefix))
         if prefix == "apply" and hasattr(self, "evaluate"):
             functions.append((Symbol(self.get_name()), self.evaluate))
         return functions
 
 
+class SympyObject(Builtin):
+    sympy_name: typing.Optional[str] = None
+
+    mathics_to_sympy = {}
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        if self.sympy_name is None:
+            self.sympy_name = strip_context(self.get_name()).lower()
+        self.mathics_to_sympy[self.__class__.__name__] = self.sympy_name
+
+    def is_constant(self) -> bool:
+        return False
+
+    def get_sympy_names(self) -> typing.List[str]:
+        if self.sympy_name:
+            return [self.sympy_name]
+        return []
+
+
 class UnaryOperator(Operator):
     def __init__(self, format_function, *args, **kwargs):
-        super(UnaryOperator, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         name = self.get_name()
         if self.needs_verbatim:
             name = "Verbatim[%s]" % name
@@ -444,12 +469,12 @@ class UnaryOperator(Operator):
 
 class PrefixOperator(UnaryOperator):
     def __init__(self, *args, **kwargs):
-        super(PrefixOperator, self).__init__("Prefix", *args, **kwargs)
+        super().__init__("Prefix", *args, **kwargs)
 
 
 class PostfixOperator(UnaryOperator):
     def __init__(self, *args, **kwargs):
-        super(PostfixOperator, self).__init__("Postfix", *args, **kwargs)
+        super().__init__("Postfix", *args, **kwargs)
 
 
 class BinaryOperator(Operator):
@@ -504,37 +529,43 @@ class Test(Builtin):
         "%(name)s[expr_]"
 
         if self.test(expr):
-            return Symbol("True")
+            return SymbolTrue
         else:
-            return Symbol("False")
-
-
-class SympyObject(Builtin):
-    sympy_name: typing.Optional[str] = None
-
-    def __init__(self, *args, **kwargs):
-        super(SympyObject, self).__init__(*args, **kwargs)
-        if self.sympy_name is None:
-            self.sympy_name = strip_context(self.get_name()).lower()
-
-    def is_constant(self) -> bool:
-        return False
-
-    def get_sympy_names(self) -> typing.List[str]:
-        if self.sympy_name:
-            return [self.sympy_name]
-        return []
+            return SymbolFalse
 
 
 class SympyFunction(SympyObject):
-    def prepare_sympy(self, leaves):
-        return leaves
+    def apply(self, *args):
+        """
+        Generic apply method that uses the class sympy_name.
+        to call the corresponding sympy function. Arguments are
+        converted to python and the result is converted from sympy
+        """
+        sympy_args = [a.to_sympy() for a in args]
+        sympy_fn = getattr(sympy, self.sympy_name)
+        return from_sympy(sympy_fn(*sympy_args))
 
-    def get_sympy_function(self, leaves):
+    def get_constant(self, precision, evaluation, have_mpmath=False):
+        try:
+            d = get_precision(precision, evaluation)
+        except PrecisionValueError:
+            return
+
+        sympy_fn = self.to_sympy()
+        if d is None:
+            result = self.get_mpmath_function() if have_mpmath else sympy_fn()
+            return MachineReal(result)
+        else:
+            return PrecisionReal(sympy_fn.n(d))
+
+    def get_sympy_function(self, leaves=None):
         if self.sympy_name:
             return getattr(sympy, self.sympy_name)
         return None
 
+    def prepare_sympy(self, leaves):
+        return leaves
+
     def to_sympy(self, expr, **kwargs):
         try:
             if self.sympy_name:
@@ -554,21 +585,6 @@ class SympyFunction(SympyObject):
         return sympy_expr
 
 
-class SympyConstant(SympyObject, Predefined):
-    attributes = ("Constant", "ReadProtected")
-
-    def is_constant(self) -> bool:
-        # free Symbol will be converted to corresponding SymPy symbol
-        return True
-
-    def to_sympy(self, expr, **kwargs):
-        if expr.is_atom():
-            return getattr(sympy, self.sympy_name)
-        else:
-            # there is no "native" SymPy expression for e.g. E[x]
-            return None
-
-
 class InvalidLevelspecError(Exception):
     pass
 
@@ -590,17 +606,111 @@ class BoxConstructError(Exception):
     pass
 
 
-class BoxConstruct(Builtin):
-    def get_option_values(self, leaves, evaluation=None, **options):
-        default = evaluation.definitions.get_options(self.get_name()).copy()
-        options = Expression("List", *leaves).get_option_values(evaluation)
-        default.update(options)
+class BoxConstruct(InstanceableBuiltin):
+    def __new__(cls, *leaves, **kwargs):
+        instance = super().__new__(cls, *leaves, **kwargs)
+        instance._leaves = leaves
+        return instance
+
+    def evaluate(self, evaluation):
+        # THINK about: Should we evaluate the leaves here?
+        return
+
+    def get_head_name(self):
+        return self.get_name()
+
+    def get_lookup_name(self):
+        return self.get_name()
+
+    def get_string_value(self):
+        return "-@" + self.get_head_name() + "@-"
+
+    def same(self, expr):
+        return expr.same(self)
+
+    def is_atom(self):
+        return False
+
+    def do_format(self, evaluation, format):
+        return self
+
+    def format(self, evaluation, fmt):
+        return self
+
+    def get_head(self):
+        return Symbol(self.get_name())
+
+    @property
+    def head(self):
+        return self.get_head()
+
+    @head.setter
+    def head(self, value):
+        raise ValueError("BoxConstruct.head is write protected.")
+
+    @property
+    def leaves(self):
+        return self._leaves
+
+    @leaves.setter
+    def leaves(self, value):
+        raise ValueError("BoxConstruct.leaves is write protected.")
+
+    # I need to repeat this, because this is not
+    # an expression...
+    def has_form(self, heads, *leaf_counts):
+        """
+        leaf_counts:
+            (,):        no leaves allowed
+            (None,):    no constraint on number of leaves
+            (n, None):  leaf count >= n
+            (n1, n2, ...):    leaf count in {n1, n2, ...}
+        """
+
+        head_name = self.get_name()
+        if isinstance(heads, (tuple, list, set)):
+            if head_name not in [ensure_context(h) for h in heads]:
+                return False
+        else:
+            if head_name != ensure_context(heads):
+                return False
+        if not leaf_counts:
+            return False
+        if leaf_counts and leaf_counts[0] is not None:
+            count = len(self._leaves)
+            if count not in leaf_counts:
+                if (
+                    len(leaf_counts) == 2
+                    and leaf_counts[1] is None  # noqa
+                    and count >= leaf_counts[0]
+                ):
+                    return True
+                else:
+                    return False
+        return True
+
+    def flatten_pattern_sequence(self, evaluation) -> "BoxConstruct":
+        return self
+
+    def get_option_values(self, leaves, **options):
+        evaluation = options.get("evaluation", None)
+        if evaluation:
+            default = evaluation.definitions.get_options(self.get_name()).copy()
+            options = Expression("List", *leaves).get_option_values(evaluation)
+            default.update(options)
+        else:
+            from mathics.core.parser import parse_builtin_rule
+
+            default = {}
+            for option, value in self.options.items():
+                option = ensure_context(option)
+                default[option] = parse_builtin_rule(value)
         return default
 
     def boxes_to_text(self, leaves, **options) -> str:
         raise BoxConstructError
 
-    def boxes_to_xml(self, leaves, **options) -> str:
+    def boxes_to_mathml(self, leaves, **options) -> str:
         raise BoxConstructError
 
     def boxes_to_tex(self, leaves, **options) -> str:
@@ -609,21 +719,21 @@ class BoxConstruct(Builtin):
 
 class PatternError(Exception):
     def __init__(self, name, tag, *args):
-        super(PatternError).__init__()
+        super().__init__()
 
 
 class PatternArgumentError(PatternError):
     def __init__(self, name, count, expected):
-        super(PatternArgumentError, self).__init__(None, None)
+        super().__init__(None, None)
 
 
-class PatternObject(InstancableBuiltin, Pattern):
+class PatternObject(InstanceableBuiltin, Pattern):
     needs_verbatim = True
 
     arg_counts: typing.List[int] = []
 
     def init(self, expr):
-        super(PatternObject, self).init(expr)
+        super().init(expr)
         if self.arg_counts is not None:
             if len(expr.leaves) not in self.arg_counts:
                 self.error_args(len(expr.leaves), *self.arg_counts)
