@@ -1,37 +1,42 @@
-#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-
+from mathics.version import __version__  # noqa used in loading to check consistency.
 from mathics.builtin.base import Builtin, Predefined
-from mathics.core.expression import (Expression, String, Symbol, Integer,
-                                     fully_qualified_symbol_name)
+from mathics.core.expression import (
+    Expression,
+    String,
+    Symbol,
+    Integer,
+    fully_qualified_symbol_name,
+)
 from mathics.core.rules import Rule
 
-def get_scoping_vars(var_list, msg_symbol='', evaluation=None):
+
+def get_scoping_vars(var_list, msg_symbol="", evaluation=None):
     def message(tag, *args):
         if msg_symbol and evaluation:
             evaluation.message(msg_symbol, tag, *args)
 
-    if not var_list.has_form('List', None):
-        message('lvlist', var_list)
+    if not var_list.has_form("List", None):
+        message("lvlist", var_list)
         return
     vars = var_list.leaves
     scoping_vars = set()
     for var in vars:
         var_name = None
-        if var.has_form('Set', 2):
+        if var.has_form("Set", 2):
             var_name = var.leaves[0].get_name()
             new_def = var.leaves[1]
             if evaluation:
                 new_def = new_def.evaluate(evaluation)
-        elif var.has_form('Symbol'):
+        elif var.has_form("Symbol"):
             var_name = var.get_name()
             new_def = None
         if not var_name:
-            message('lvsym', var)
+            message("lvsym", var)
             continue
         if var_name in scoping_vars:
-            message('dup', Symbol(var_name))
+            message("dup", Symbol(var_name))
         else:
             scoping_vars.add(var_name)
             yield var_name, new_def
@@ -41,8 +46,9 @@ def dynamic_scoping(func, vars, evaluation):
     original_definitions = {}
     for var_name, new_def in vars.items():
         assert fully_qualified_symbol_name(var_name)
-        original_definitions[
-            var_name] = evaluation.definitions.get_user_definition(var_name)
+        original_definitions[var_name] = evaluation.definitions.get_user_definition(
+            var_name
+        )
         evaluation.definitions.reset_user_definition(var_name)
         if new_def is not None:
             new_def = new_def.evaluate(evaluation)
@@ -89,20 +95,23 @@ class With(Builtin):
 
     """
 
-    attributes = ('HoldAll',)
+    attributes = ("HoldAll",)
 
     messages = {
-        'lvsym': ("Local variable specification contains `1`, "
-                  "which is not a symbol or an assignment to a symbol."),
-        'dup': ("Duplicate local variable `1` found in local variable "
-                "specification."),
-        'lvlist': "Local variable specification `1` is not a List.",
+        "lvsym": (
+            "Local variable specification contains `1`, "
+            "which is not a symbol or an assignment to a symbol."
+        ),
+        "dup": (
+            "Duplicate local variable `1` found in local variable " "specification."
+        ),
+        "lvlist": "Local variable specification `1` is not a List.",
     }
 
     def apply(self, vars, expr, evaluation):
-        'With[vars_, expr_]'
+        "With[vars_, expr_]"
 
-        vars = dict(get_scoping_vars(vars, 'With', evaluation))
+        vars = dict(get_scoping_vars(vars, "With", evaluation))
         result = expr.replace_vars(vars)
         result.evaluate(evaluation)
         return result
@@ -144,20 +153,23 @@ class Block(Builtin):
      = x
     """
 
-    attributes = ('HoldAll',)
+    attributes = ("HoldAll",)
 
     messages = {
-        'lvsym': ("Local variable specification contains `1`, "
-                  "which is not a symbol or an assignment to a symbol."),
-        'dup': ("Duplicate local variable `1` found in local variable "
-                "specification."),
-        'lvlist': "Local variable specification `1` is not a List.",
+        "lvsym": (
+            "Local variable specification contains `1`, "
+            "which is not a symbol or an assignment to a symbol."
+        ),
+        "dup": (
+            "Duplicate local variable `1` found in local variable " "specification."
+        ),
+        "lvlist": "Local variable specification `1` is not a List.",
     }
 
     def apply(self, vars, expr, evaluation):
-        'Block[vars_, expr_]'
+        "Block[vars_, expr_]"
 
-        vars = dict(get_scoping_vars(vars, 'Block', evaluation))
+        vars = dict(get_scoping_vars(vars, "Block", evaluation))
         result = dynamic_scoping(expr.evaluate, vars, evaluation)
         return result
 
@@ -178,15 +190,16 @@ class ModuleNumber(Predefined):
      : Cannot set $ModuleNumber to x; value must be a positive integer.
     """
 
-    name = '$ModuleNumber'
+    name = "$ModuleNumber"
 
     messages = {
-        'set': ("Cannot set $ModuleNumber to `1`; "
-                "value must be a positive integer."),
+        "set": (
+            "Cannot set $ModuleNumber to `1`; " "value must be a positive integer."
+        ),
     }
 
     rules = {
-        '$ModuleNumber': '1',
+        "$ModuleNumber": "1",
     }
 
 
@@ -228,28 +241,30 @@ class Module(Builtin):
 
     """
 
-    attributes = ('HoldAll',)
+    attributes = ("HoldAll",)
 
     messages = {
-        'lvsym': ("Local variable specification contains `1`, "
-                  "which is not a symbol or an assignment to a symbol."),
-        'dup': ("Duplicate local variable `1` found in local variable "
-                "specification."),
-        'lvlist': "Local variable specification `1` is not a List.",
+        "lvsym": (
+            "Local variable specification contains `1`, "
+            "which is not a symbol or an assignment to a symbol."
+        ),
+        "dup": (
+            "Duplicate local variable `1` found in local variable " "specification."
+        ),
+        "lvlist": "Local variable specification `1` is not a List.",
     }
 
     def apply(self, vars, expr, evaluation):
-        'Module[vars_, expr_]'
+        "Module[vars_, expr_]"
 
-        scoping_vars = get_scoping_vars(vars, 'Module', evaluation)
+        scoping_vars = get_scoping_vars(vars, "Module", evaluation)
         replace = {}
-        number = Symbol('$ModuleNumber').evaluate(evaluation).get_int_value()
+        number = Symbol("$ModuleNumber").evaluate(evaluation).get_int_value()
         if number is None:
             number = 1
-        evaluation.definitions.set_ownvalue(
-            '$ModuleNumber', Integer(number + 1))
+        evaluation.definitions.set_ownvalue("$ModuleNumber", Integer(number + 1))
         for name, new_def in scoping_vars:
-            new_name = '%s$%d' % (name, number)
+            new_name = "%s$%d" % (name, number)
             if new_def is not None:
                 evaluation.definitions.set_ownvalue(new_name, new_def)
             replace[name] = Symbol(new_name)
@@ -334,50 +349,52 @@ class Unique(Predefined):
     seq_number = 1
 
     messages = {
-        'usym': '`1` is not a symbol or a valid symbol name.',
-        'argrx': 'Unique called with `1` arguments; 0 or 1 argument are expected.',
-        'attnf': '`1` is not a known attribute.',
+        "usym": "`1` is not a symbol or a valid symbol name.",
+        "argrx": "Unique called with `1` arguments; 0 or 1 argument are expected.",
+        "attnf": "`1` is not a known attribute.",
     }
 
-    attributes = ('Protected',)
+    attributes = ("Protected",)
 
     rules = {
-        'Unique[x_Symbol]': 'Module[{x}, x]',
+        "Unique[x_Symbol]": "Module[{x}, x]",
     }
 
     def apply(self, evaluation):
-        'Unique[]'
+        "Unique[]"
 
-        new_name = '$%d' % (self.seq_number)
+        new_name = "$%d" % (self.seq_number)
         self.seq_number += 1
         # Next symbol in case of new name is defined before
         while evaluation.definitions.get_definition(new_name, True) is not None:
-            new_name = '$%d' % (self.seq_number)
+            new_name = "$%d" % (self.seq_number)
             self.seq_number += 1
         return Symbol(new_name)
 
     def apply_symbol(self, vars, attributes, evaluation):
-        'Unique[vars_, attributes___]'
+        "Unique[vars_, attributes___]"
 
         from mathics.core.parser import is_symbol_name
         from mathics.builtin.attributes import get_symbol_list
 
         attributes = attributes.get_sequence()
         if len(attributes) > 1:
-            return evaluation.message('Unique', 'argrx', Integer(len(attributes) + 1))
+            return evaluation.message("Unique", "argrx", Integer(len(attributes) + 1))
 
         # Check valid symbol variables
-        symbols = vars.leaves if vars.has_form('List', None) else [vars]
+        symbols = vars.leaves if vars.has_form("List", None) else [vars]
         for symbol in symbols:
             if not isinstance(symbol, Symbol):
                 text = symbol.get_string_value()
                 if text is None or not is_symbol_name(text):
-                    return evaluation.message('Unique', 'usym', symbol)
+                    return evaluation.message("Unique", "usym", symbol)
 
         # Check valid attributes
         attrs = []
         if len(attributes) > 0:
-            attrs = get_symbol_list(attributes[0], lambda item: evaluation.message('Unique', 'attnf', item))
+            attrs = get_symbol_list(
+                attributes[0], lambda item: evaluation.message("Unique", "attnf", item)
+            )
             if attrs is None:
                 return None
 
@@ -385,21 +402,23 @@ class Unique(Predefined):
         list = []
         for symbol in symbols:
             if isinstance(symbol, Symbol):
-                list.append(Module(Expression('List', symbol), symbol).evaluate(evaluation))
+                list.append(
+                    Module(Expression("List", symbol), symbol).evaluate(evaluation)
+                )
             else:
-                new_name = '%s%d' % (symbol.get_string_value(), self.seq_number)
+                new_name = "%s%d" % (symbol.get_string_value(), self.seq_number)
                 self.seq_number += 1
                 # Next symbol in case of new name is defined before
                 while evaluation.definitions.get_definition(new_name, True) is not None:
-                    new_name = '%s%d' % (symbol.get_string_value(), self.seq_number)
+                    new_name = "%s%d" % (symbol.get_string_value(), self.seq_number)
                     self.seq_number += 1
                 list.append(Symbol(new_name))
         for symbol in list:
             for att in attrs:
                 evaluation.definitions.set_attribute(symbol.get_name(), att)
 
-        if vars.has_form('List', None):
-            return Expression('List', *list)
+        if vars.has_form("List", None):
+            return Expression("List", *list)
         else:
             return list[0]
 
@@ -437,21 +456,19 @@ class Context(Builtin):
      = Pattern[a`b, Blank[]]
     """
 
-    attributes = ('HoldFirst',)
+    attributes = ("HoldFirst",)
 
-    rules = {
-        'Context[]': '$Context'
-    }
+    rules = {"Context[]": "$Context"}
 
     def apply(self, symbol, evaluation):
-        'Context[symbol_]'
+        "Context[symbol_]"
 
         name = symbol.get_name()
         if not name:
-            evaluation.message('Context', 'normal')
+            evaluation.message("Context", "normal")
             return
-        assert '`' in name
-        context = name[:name.rindex('`') + 1]
+        assert "`" in name
+        context = name[: name.rindex("`") + 1]
         return String(context)
 
 
@@ -468,13 +485,13 @@ class Contexts(Builtin):
     """
 
     def apply(self, evaluation):
-        'Contexts[]'
+        "Contexts[]"
 
         contexts = set([])
         for name in evaluation.definitions.get_names():
-            contexts.add(String(name[:name.rindex('`') + 1]))
+            contexts.add(String(name[: name.rindex("`") + 1]))
 
-        return Expression('List', *sorted(contexts))
+        return Expression("List", *sorted(contexts))
 
 
 class Context_(Predefined):
@@ -497,14 +514,12 @@ class Context_(Predefined):
      = True
     """
 
-    name = '$Context'
+    name = "$Context"
 
-    messages = {
-        'cxset': "`1` is not a valid context name ending in `."
-    }
+    messages = {"cxset": "`1` is not a valid context name ending in `."}
 
     rules = {
-        '$Context': '"Global`"',
+        "$Context": '"Global`"',
     }
 
 
@@ -527,14 +542,12 @@ class ContextPath(Predefined):
     #> $ContextPath = {"Global`", "System`"};
     """
 
-    name = '$ContextPath'
+    name = "$ContextPath"
 
-    messages = {
-        'cxlist': "`1` is not a list of valid context names ending in `."
-    }
+    messages = {"cxlist": "`1` is not a list of valid context names ending in `."}
 
     rules = {
-        '$ContextPath': '{"Global`", "System`"}',
+        "$ContextPath": '{"Global`", "System`"}',
     }
 
 
@@ -565,13 +578,13 @@ class Begin(Builtin):
     """
 
     rules = {
-        'Begin[context_String]': '''
+        "Begin[context_String]": """
              Unprotect[System`Private`$ContextStack];
              System`Private`$ContextStack = Append[System`Private`$ContextStack, $Context];
              Protect[System`Private`$ContextStack];
              $Context = context;
              $Context
-        ''',
+        """,
     }
 
 
@@ -584,11 +597,11 @@ class End(Builtin):
     """
 
     messages = {
-        'noctx': "No previous context defined.",
+        "noctx": "No previous context defined.",
     }
 
     rules = {
-        'End[]': '''
+        "End[]": """
              Block[{System`Private`old=$Context},
                    If[Length[System`Private`$ContextStack] === 0,
                      (* then *) Message[End::noctx]; $Context,
@@ -598,7 +611,7 @@ class End(Builtin):
                                      Most[System`Private`$ContextStack]};
                                 Protect[System`Private`$ContextStack];
                                 System`Private`old]]
-        ''',
+        """,
     }
 
 
@@ -617,23 +630,21 @@ class BeginPackage(Builtin):
      = test`
     """
 
-    messages = {
-        'unimpl': "The second argument to BeginPackage is not yet implemented."
-    }
+    messages = {"unimpl": "The second argument to BeginPackage is not yet implemented."}
 
     rules = {
-        'BeginPackage[context_String]': '''
+        "BeginPackage[context_String]": """
              Unprotect[System`Private`$ContextPathStack, System`$Packages];
              Begin[context];
              System`Private`$ContextPathStack =
                  Append[System`Private`$ContextPathStack, $ContextPath];
              $ContextPath = {context, "System`"};
              $Packages = If[MemberQ[System`$Packages,$Context],
-                            None,
+                            $Packages,
                             System`$Packages=Join[{$Context}, System`$Packages]];
              Protect[System`Private`$ContextPathStack, System`$Packages];
              context
-        ''',
+        """,
     }
 
 
@@ -650,11 +661,11 @@ class EndPackage(Builtin):
     """
 
     messages = {
-        'noctx': "No previous context defined.",
+        "noctx": "No previous context defined.",
     }
 
     rules = {
-        'EndPackage[]': '''
+        "EndPackage[]": """
              Block[{System`Private`newctx=Quiet[End[], {End::noctx}]},
                    If[Length[System`Private`$ContextPathStack] === 0,
                       (* then *) Message[EndPackage::noctx],
@@ -665,7 +676,7 @@ class EndPackage(Builtin):
                                       Most[System`Private`$ContextPathStack]};
                                  Protect[System`Private`$ContextPathStack];
                                  Null]]
-        ''',
+        """,
     }
 
 
@@ -678,11 +689,11 @@ class ContextStack(Builtin):
     </dl>
     """
 
-    context = 'System`Private`'
-    name = '$ContextStack'
+    context = "System`Private`"
+    name = "$ContextStack"
 
     rules = {
-        'System`Private`$ContextStack': '{}',
+        "System`Private`$ContextStack": "{}",
     }
 
 
@@ -695,9 +706,9 @@ class ContextPathStack(Builtin):
     </dl>
     """
 
-    context = 'System`Private`'
-    name = '$ContextPathStack'
+    context = "System`Private`"
+    name = "$ContextPathStack"
 
     rules = {
-        'System`Private`$ContextPathStack': '{}',
+        "System`Private`$ContextPathStack": "{}",
     }
