@@ -20,12 +20,12 @@ machine_epsilon = 2 ** (1 - machine_precision)
 
 
 def reconstruct_digits(bits) -> int:
-    '''
+    """
     Number of digits needed to reconstruct a number with given bits of precision.
 
     >>> reconstruct_digits(53)
     17
-    '''
+    """
     return int(ceil(bits / C) + 1)
 
 
@@ -40,34 +40,35 @@ class SpecialValueError(Exception):
 
 def _get_float_inf(value, evaluation) -> typing.Optional[float]:
     value = value.evaluate(evaluation)
-    if value.has_form('DirectedInfinity', 1):
+    if value.has_form("DirectedInfinity", 1):
         if value.leaves[0].get_int_value() == 1:
-            return float('inf')
+            return float("inf")
         elif value.leaves[0].get_int_value() == -1:
-            return float('-inf')
+            return float("-inf")
         else:
             return None
     return value.round_to_float(evaluation)
 
 
 def get_precision(value, evaluation) -> typing.Optional[int]:
-    if value.get_name() == 'System`MachinePrecision':
+    if value.get_name() == "System`MachinePrecision":
         return None
     else:
         from mathics.core.expression import Symbol, MachineReal
-        dmin = _get_float_inf(Symbol('$MinPrecision'), evaluation)
-        dmax = _get_float_inf(Symbol('$MaxPrecision'), evaluation)
+
+        dmin = _get_float_inf(Symbol("$MinPrecision"), evaluation)
+        dmax = _get_float_inf(Symbol("$MaxPrecision"), evaluation)
         d = value.round_to_float(evaluation)
         assert dmin is not None and dmax is not None
         if d is None:
-            evaluation.message('N', 'precbd', value)
+            evaluation.message("N", "precbd", value)
         elif d < dmin:
             dmin = int(dmin)
-            evaluation.message('N', 'precsm', value, MachineReal(dmin))
+            evaluation.message("N", "precsm", value, MachineReal(dmin))
             return dmin
         elif d > dmax:
             dmax = int(dmax)
-            evaluation.message('N', 'preclg', value, MachineReal(dmax))
+            evaluation.message("N", "preclg", value, MachineReal(dmax))
             return dmax
         else:
             return d
@@ -76,14 +77,15 @@ def get_precision(value, evaluation) -> typing.Optional[int]:
 
 def get_type(value) -> typing.Optional[str]:
     if isinstance(value, sympy.Integer):
-        return 'z'
+        return "z"
     elif isinstance(value, sympy.Rational):
-        return 'q'
+        return "q"
     elif isinstance(value, sympy.Float) or isinstance(value, mpmath.mpf):
-        return 'f'
-    elif (isinstance(value, sympy.Expr) and value.is_number and
-          not value.is_real) or isinstance(value, mpmath.mpc):
-        return 'c'
+        return "f"
+    elif (
+        isinstance(value, sympy.Expr) and value.is_number and not value.is_real
+    ) or isinstance(value, mpmath.mpc):
+        return "c"
     else:
         return None
 
@@ -115,15 +117,16 @@ def pickle_mp(value):
 
 def unpickle_mp(value):
     type, value = value
-    if type == 'z':
+    if type == "z":
         return sympy.Integer(value)
-    elif type == 'q':
+    elif type == "q":
         return sympy.Rational(value)
-    elif type == 'f':
+    elif type == "f":
         return sympy.Float(value)
     else:
         return value
 
+
 # algorithm based on
 # http://stackoverflow.com/questions/5110177/how-to-convert-floating-point-number-to-base-3-in-python       # nopep8
 
@@ -151,15 +154,15 @@ def convert_base(x, base, precision=10) -> str:
 
     int_part = convert(int(x), base, iexps)
     if sign == -1:
-        int_part.insert(0, '-')
+        int_part.insert(0, "-")
 
     if isinstance(x, (float, sympy.Float)):
         fexps = list(range(-1, -int(precision + 1), -1))
         real_part = convert(x - int(x), base, fexps)
 
-        return "%s.%s" % (''.join(int_part), ''.join(real_part))
+        return "%s.%s" % ("".join(int_part), "".join(real_part))
     elif isinstance(x, int):
-        return ''.join(int_part)
+        return "".join(int_part)
     else:
         raise TypeError(x)
 
@@ -178,7 +181,7 @@ def convert_int_to_digit_list(x, base) -> typing.List[int]:
         for e in exponents:
             d = int(x // (base ** e))
             x -= d * (base ** e)
-            if out or d != 0:   # drop any leading zeroes
+            if out or d != 0:  # drop any leading zeroes
                 out.append(d)
             if x == 0 and e < 0:
                 break
