@@ -13,31 +13,49 @@ from mathics_scanner import (
 
 from mathics.core.parser.ast import Node, Number, Symbol, String, Filename
 from mathics.core.parser.operators import (
-    prefix_ops, postfix_ops, left_binary_ops, right_binary_ops,
-    nonassoc_binary_ops, flat_binary_ops, ternary_ops, binary_ops, all_ops,
-    inequality_ops, misc_ops)
+    prefix_ops,
+    postfix_ops,
+    left_binary_ops,
+    right_binary_ops,
+    nonassoc_binary_ops,
+    flat_binary_ops,
+    ternary_ops,
+    binary_ops,
+    all_ops,
+    inequality_ops,
+    misc_ops,
+)
 
 
 special_symbols = {
-    '\u03C0': 'Pi',         # Pi
-    '\uF74D': 'E',          # ExponentialE
-    '\uF74E': 'I',          # ImaginaryI
-    '\uF74F': 'I',          # ImaginaryJ
-    '\u221E': 'Infinity',   # Infinity
-    '\u00B0': 'Degree',     # Degree
+    "\u03C0": "Pi",  # Pi
+    "\uF74D": "E",  # ExponentialE
+    "\uF74E": "I",  # ImaginaryI
+    "\uF74F": "I",  # ImaginaryJ
+    "\u221E": "Infinity",  # Infinity
+    "\u00B0": "Degree",  # Degree
 }
 
 
 permitted_digits = {c: i for i, c in enumerate(string.digits + string.ascii_lowercase)}
-permitted_digits['.'] = 0
+permitted_digits["."] = 0
 
 
 class Parser(object):
     def __init__(self):
         # no implicit times on these tokens
-        self.halt_tags = set([
-            'END', 'RawRightAssociation', 'RawRightParenthesis', 'RawComma', 'RawRightBrace',
-            'RawRightBracket', 'RawColon', 'DifferentialD'])
+        self.halt_tags = set(
+            [
+                "END",
+                "RawRightAssociation",
+                "RawRightParenthesis",
+                "RawComma",
+                "RawRightBrace",
+                "RawRightBracket",
+                "RawColon",
+                "DifferentialD",
+            ]
+        )
 
     def parse(self, feeder):
         self.feeder = feeder
@@ -53,10 +71,10 @@ class Parser(object):
         return self.current_token
 
     def next_noend(self):
-        'returns next token which is not END'
+        "returns next token which is not END"
         while True:
             token = self.next()
-            if token.tag != 'END':
+            if token.tag != "END":
                 return token
             self.incomplete(token.pos)
 
@@ -81,10 +99,10 @@ class Parser(object):
 
     def parse_e(self):
         result = []
-        while self.next().tag != 'END':
+        while self.next().tag != "END":
             result.append(self.parse_exp(0))
         if len(result) > 1:
-            return Node('Times', *result)
+            return Node("Times", *result)
         if len(result) == 1:
             return result[0]
         else:
@@ -98,7 +116,7 @@ class Parser(object):
             else:
                 token = self.next()
             tag = token.tag
-            method = getattr(self, 'e_' + tag, None)
+            method = getattr(self, "e_" + tag, None)
             if method is not None:
                 new_result = method(result, token, p)
             elif tag in inequality_ops:
@@ -109,11 +127,11 @@ class Parser(object):
                 new_result = self.parse_ternary(result, token, p)
             elif tag in postfix_ops:
                 new_result = self.parse_postfix(result, token, p)
-            elif tag not in self.halt_tags and flat_binary_ops['Times'] >= p:
+            elif tag not in self.halt_tags and flat_binary_ops["Times"] >= p:
                 # implicit times
-                q = flat_binary_ops['Times']
+                q = flat_binary_ops["Times"]
                 child = self.parse_exp(q + 1)
-                new_result = Node('Times', result, child).flatten()
+                new_result = Node("Times", result, child).flatten()
             else:
                 new_result = None
             if new_result is None:
@@ -125,7 +143,7 @@ class Parser(object):
     def parse_p(self):
         token = self.next_noend()
         tag = token.tag
-        method = getattr(self, 'p_' + tag, None)
+        method = getattr(self, "p_" + tag, None)
         if method is not None:
             return method(token)
         elif tag in prefix_ops:
@@ -142,14 +160,14 @@ class Parser(object):
         while True:
             token = self.next()
             tag = token.tag
-            method = getattr(self, 'b_' + tag, None)
+            method = getattr(self, "b_" + tag, None)
             if method is not None:
                 new_result = method(result, token, p)
-            elif tag in ('OtherscriptBox', 'RightRowBox'):
+            elif tag in ("OtherscriptBox", "RightRowBox"):
                 break
-            elif tag == 'END':
+            elif tag == "END":
                 self.incomplete(token.pos)
-            elif result is None and tag != 'END':
+            elif result is None and tag != "END":
                 self.consume()
                 new_result = String(token.text)
             else:
@@ -159,7 +177,7 @@ class Parser(object):
             else:
                 result = new_result
         if result is None:
-            result = String('')
+            result = String("")
         return result
 
     def parse_seq(self):
@@ -167,23 +185,23 @@ class Parser(object):
         while True:
             token = self.next_noend()
             tag = token.tag
-            if tag == 'RawComma':
-                self.tokeniser.feeder.message('Syntax', 'com')
-                result.append(Symbol('Null'))
+            if tag == "RawComma":
+                self.tokeniser.feeder.message("Syntax", "com")
+                result.append(Symbol("Null"))
                 self.consume()
-            elif tag in ('RawRightAssociation', 'RawRightBrace', 'RawRightBracket'):
+            elif tag in ("RawRightAssociation", "RawRightBrace", "RawRightBracket"):
                 if result:
-                    self.tokeniser.feeder.message('Syntax', 'com')
-                    result.append(Symbol('Null'))
+                    self.tokeniser.feeder.message("Syntax", "com")
+                    result.append(Symbol("Null"))
                 break
             else:
                 result.append(self.parse_exp(0))
                 token = self.next_noend()
                 tag = token.tag
-                if tag == 'RawComma':
+                if tag == "RawComma":
                     self.consume()
                     continue
-                elif tag in ('RawRightAssociation', 'RawRightBrace', 'RawRightBracket'):
+                elif tag in ("RawRightAssociation", "RawRightBrace", "RawRightBracket"):
                     break
         return result
 
@@ -195,7 +213,7 @@ class Parser(object):
         self.consume()
         head = expr1.get_head_name()
         expr2 = self.parse_exp(q + 1)
-        if head == 'Inequality' and not expr1.parenthesised:
+        if head == "Inequality" and not expr1.parenthesised:
             expr1.children.append(Symbol(tag))
             expr1.children.append(expr2)
         elif head in inequality_ops and head != tag and not expr1.parenthesised:
@@ -208,7 +226,7 @@ class Parser(object):
                 first = False
             children.append(Symbol(tag))
             children.append(expr2)
-            expr1 = Node('Inequality', *children)
+            expr1 = Node("Inequality", *children)
         else:
             expr1 = Node(tag, expr1, expr2).flatten()
         return expr1
@@ -223,7 +241,11 @@ class Parser(object):
             q += 1
         expr2 = self.parse_exp(q)
         # flatten or associate
-        if tag in nonassoc_binary_ops and expr1.get_head_name() == tag and not expr1.parenthesised:
+        if (
+            tag in nonassoc_binary_ops
+            and expr1.get_head_name() == tag
+            and not expr1.parenthesised
+        ):
             self.tokeniser.sntx_message(token.pos)
             raise InvalidSyntaxError()
         result = Node(tag, expr1, expr2)
@@ -247,21 +269,21 @@ class Parser(object):
 
     def p_Factorial(self, token):
         self.consume()
-        q = prefix_ops['Not']
+        q = prefix_ops["Not"]
         child = self.parse_exp(q)
-        return Node('Not', child)
+        return Node("Not", child)
 
     def p_Factorial2(self, token):
         self.consume()
-        q = prefix_ops['Not']
+        q = prefix_ops["Not"]
         child = self.parse_exp(q)
-        return Node('Not', Node('Not', child))
+        return Node("Not", Node("Not", child))
 
     def p_RawLeftParenthesis(self, token):
         self.consume()
         self.bracket_depth += 1
         result = self.parse_exp(0)
-        self.expect('RawRightParenthesis')
+        self.expect("RawRightParenthesis")
         self.bracket_depth -= 1
         result.parenthesised = True
         return result
@@ -270,17 +292,17 @@ class Parser(object):
         self.consume()
         self.bracket_depth += 1
         seq = self.parse_seq()
-        self.expect('RawRightBrace')
+        self.expect("RawRightBrace")
         self.bracket_depth -= 1
-        return Node('List', *seq)
+        return Node("List", *seq)
 
     def p_RawLeftAssociation(self, token):
         self.consume()
         self.bracket_depth += 1
         seq = self.parse_seq()
-        self.expect('RawRightAssociation')
+        self.expect("RawRightAssociation")
         self.bracket_depth -= 1
-        return Node('Association', *seq)
+        return Node("Association", *seq)
 
     def p_LeftRowBox(self, token):
         self.consume()
@@ -288,16 +310,16 @@ class Parser(object):
         self.box_depth += 1
         self.bracket_depth += 1
         token = self.next()
-        while token.tag not in ('RightRowBox', 'OtherscriptBox'):
+        while token.tag not in ("RightRowBox", "OtherscriptBox"):
             children.append(self.parse_box(0))
             token = self.next()
         if len(children) == 0:
-            result = String('')
+            result = String("")
         elif len(children) == 1:
             result = children[0]
         else:
-            result = Node('RowBox', Node('List', *children))
-        self.expect('RightRowBox')
+            result = Node("RowBox", Node("List", *children))
+        self.expect("RightRowBox")
         self.box_depth -= 1
         self.bracket_depth -= 1
         result.parenthesised = True
@@ -307,26 +329,26 @@ class Parser(object):
         s = token.text
 
         # sign
-        if s[0] == '-':
+        if s[0] == "-":
             s = s[1:]
             sign = -1
         else:
             sign = 1
 
         # base
-        s = s.split('^^')
+        s = s.split("^^")
         if len(s) == 1:
             base, s = 10, s[0]
         else:
             assert len(s) == 2
             base, s = int(s[0]), s[1]
             if not 2 <= base <= 36:
-                self.tokeniser.feeder.message('General', 'base', base, token.text, 36)
+                self.tokeniser.feeder.message("General", "base", base, token.text, 36)
                 self.tokeniser.sntx_message(token.pos)
                 raise InvalidSyntaxError()
 
         # mantissa
-        s = s.split('*^')
+        s = s.split("*^")
         if len(s) == 1:
             exp, s = 0, s[0]
         else:
@@ -334,7 +356,7 @@ class Parser(object):
             exp, s = int(s[1]), s[0]
 
         # precision/accuracy
-        s = s.split('`', 1)
+        s = s.split("`", 1)
         if len(s) == 1:
             s, suffix = s[0], None
         else:
@@ -342,7 +364,7 @@ class Parser(object):
 
         for i, c in enumerate(s.lower()):
             if permitted_digits[c] >= base:
-                self.tokeniser.feeder.message('General', 'digit', i + 1, s, base)
+                self.tokeniser.feeder.message("General", "digit", i + 1, s, base)
                 self.tokeniser.sntx_message(token.pos)
                 raise InvalidSyntaxError()
 
@@ -367,124 +389,131 @@ class Parser(object):
         return result
 
     def p_Span(self, token):
-        return self.e_Span(Number('1'), token, 0)
+        return self.e_Span(Number("1"), token, 0)
 
     def p_Integral(self, token):
         self.consume()
-        inner_prec, outer_prec = all_ops['Sum'] + 1, all_ops['Power'] - 1
+        inner_prec, outer_prec = all_ops["Sum"] + 1, all_ops["Power"] - 1
         expr1 = self.parse_exp(inner_prec)
-        self.expect('DifferentialD')
+        self.expect("DifferentialD")
         expr2 = self.parse_exp(outer_prec)
-        return Node('Integrate', expr1, expr2)
+        return Node("Integrate", expr1, expr2)
 
     def p_Pattern(self, token):
         self.consume()
         text = token.text
-        if '.' in text:
+        if "." in text:
             name = text[:-2]
             if name:
-                return Node('Optional', Node('Pattern', Symbol(name, context=None), Node('Blank')))
+                return Node(
+                    "Optional",
+                    Node("Pattern", Symbol(name, context=None), Node("Blank")),
+                )
             else:
-                return Node('Optional', Node('Blank'))
-        pieces = text.split('_')
+                return Node("Optional", Node("Blank"))
+        pieces = text.split("_")
         count = len(pieces) - 1
         if count == 1:
-            name = 'Blank'
+            name = "Blank"
         elif count == 2:
-            name = 'BlankSequence'
+            name = "BlankSequence"
         elif count == 3:
-            name = 'BlankNullSequence'
+            name = "BlankNullSequence"
         if pieces[-1]:
             blank = Node(name, Symbol(pieces[-1], context=None))
         else:
             blank = Node(name)
         if pieces[0]:
-            return Node('Pattern', Symbol(pieces[0], context=None), blank)
+            return Node("Pattern", Symbol(pieces[0], context=None), blank)
         else:
             return blank
 
     def p_Minus(self, token):
         self.consume()
-        q = prefix_ops['Minus']
+        q = prefix_ops["Minus"]
         expr = self.parse_exp(q)
-        if isinstance(expr, Number) and not expr.value.startswith('-'):
-            expr.value = '-' + expr.value
+        if isinstance(expr, Number) and not expr.value.startswith("-"):
+            expr.value = "-" + expr.value
             return expr
         else:
-            return Node('Times', Number('1', sign=-1), expr).flatten()
+            return Node("Times", Number("1", sign=-1), expr).flatten()
 
     def p_Plus(self, token):
         self.consume()
-        q = prefix_ops['Minus']
+        q = prefix_ops["Minus"]
         # note flattening here even flattens e.g. + a + b
-        return Node('Plus', self.parse_exp(q)).flatten()
+        return Node("Plus", self.parse_exp(q)).flatten()
 
     def p_PlusMinus(self, token):
         self.consume()
-        q = prefix_ops['Minus']
-        return Node('PlusMinus', self.parse_exp(q))
+        q = prefix_ops["Minus"]
+        return Node("PlusMinus", self.parse_exp(q))
 
     def p_MinusPlus(self, token):
         self.consume()
-        q = prefix_ops['Minus']
-        return Node('MinusPlus', self.parse_exp(q))
+        q = prefix_ops["Minus"]
+        return Node("MinusPlus", self.parse_exp(q))
 
     def p_Out(self, token):
         self.consume()
         text = token.text
-        if text == '%':
-            return Node('Out')
-        if text.endswith('%'):
+        if text == "%":
+            return Node("Out")
+        if text.endswith("%"):
             n = str(-len(text))
         else:
             n = text[1:]
-        return Node('Out', Number(n))
+        return Node("Out", Number(n))
 
     def p_Slot(self, token):
         self.consume()
         text = token.text
         if len(text) == 1:
-            n = Number('1')
+            n = Number("1")
         else:
             n = text[1:]
             if n.isdigit():
                 n = Number(n)
             else:
                 n = String(n)
-        return Node('Slot', n)
+        return Node("Slot", n)
 
     def p_SlotSequence(self, token):
         self.consume()
         text = token.text
         if len(text) == 2:
-            n = '1'
+            n = "1"
         else:
             n = text[2:]
-        return Node('SlotSequence', Number(n))
+        return Node("SlotSequence", Number(n))
 
     def p_Increment(self, token):
         self.consume()
-        q = prefix_ops['PreIncrement']
-        return Node('PreIncrement', self.parse_exp(q))
+        q = prefix_ops["PreIncrement"]
+        return Node("PreIncrement", self.parse_exp(q))
 
     def p_Decrement(self, token):
         self.consume()
-        q = prefix_ops['PreDecrement']
-        return Node('PreDecrement', self.parse_exp(q))
+        q = prefix_ops["PreDecrement"]
+        return Node("PreDecrement", self.parse_exp(q))
 
     def p_PatternTest(self, token):
         self.consume()
-        q = prefix_ops['Definition']
+        q = prefix_ops["Definition"]
         child = self.parse_exp(q)
-        return Node('Information', child, Node('Rule', Symbol("LongForm"), Symbol("False")))
+        return Node(
+            "Information", child, Node("Rule", Symbol("LongForm"), Symbol("False"))
+        )
 
     def p_Information(self, token):
         self.consume()
-        q = prefix_ops['Information']
+        q = prefix_ops["Information"]
         child = self.parse_exp(q)
         if child.__class__ is not Symbol:
             raise InvalidSyntaxError()
-        return Node('Information', child, Node('Rule', Symbol("LongForm"), Symbol("True")))
+        return Node(
+            "Information", child, Node("Rule", Symbol("LongForm"), Symbol("True"))
+        )
 
     # E methods
     #
@@ -494,75 +523,75 @@ class Parser(object):
     # return None if precedence is too low.
 
     def e_Span(self, expr1, token, p):
-        q = ternary_ops['Span']
+        q = ternary_ops["Span"]
         if q < p:
             return None
 
-        if expr1.get_head_name() == 'Span' and not expr1.parenthesised:
+        if expr1.get_head_name() == "Span" and not expr1.parenthesised:
             return None
         self.consume()
         # Span[expr1, expr2]
         token = self.next()
-        if token.tag == 'Span':
-            expr2 = Symbol('All')
-        elif token.tag == 'END' and self.bracket_depth == 0:
+        if token.tag == "Span":
+            expr2 = Symbol("All")
+        elif token.tag == "END" and self.bracket_depth == 0:
             # So that e.g. 'x = 1 ;;' doesn't wait for newline in the frontend
-            expr2 = Symbol('All')
-            return Node('Span', expr1, expr2)
+            expr2 = Symbol("All")
+            return Node("Span", expr1, expr2)
         else:
             messages = list(self.feeder.messages)
             try:
                 expr2 = self.parse_exp(q + 1)
             except TranslateError:
-                expr2 = Symbol('All')
+                expr2 = Symbol("All")
                 self.backtrack(token.pos)
                 self.feeder.messages = messages
         token = self.next()
-        if token.tag == 'Span':
+        if token.tag == "Span":
             self.consume()
             messages = list(self.feeder.messages)
             try:
                 expr3 = self.parse_exp(q + 1)
-                return Node('Span', expr1, expr2, expr3)
+                return Node("Span", expr1, expr2, expr3)
             except TranslateError:
                 self.backtrack(token.pos)
                 self.feeder.messages = messages
-        return Node('Span', expr1, expr2)
+        return Node("Span", expr1, expr2)
 
     def e_RawLeftBracket(self, expr, token, p):
-        q = all_ops['Part']
+        q = all_ops["Part"]
         if q < p:
             return None
         self.consume()
         self.bracket_depth += 1
         token = self.next_noend()
-        if token.tag == 'RawLeftBracket':
+        if token.tag == "RawLeftBracket":
             self.consume()
             seq = self.parse_seq()
-            self.expect('RawRightBracket')
-            self.expect('RawRightBracket')
+            self.expect("RawRightBracket")
+            self.expect("RawRightBracket")
             self.bracket_depth -= 1
-            return Node('Part', expr, *seq)
+            return Node("Part", expr, *seq)
         else:
             seq = self.parse_seq()
-            self.expect('RawRightBracket')
+            self.expect("RawRightBracket")
             self.bracket_depth -= 1
             result = Node(expr, *seq)
             result.parenthesised = True
             return result
 
     def e_Infix(self, expr1, token, p):
-        q = ternary_ops['Infix']
+        q = ternary_ops["Infix"]
         if q < p:
             return None
         self.consume()
         expr2 = self.parse_exp(q + 1)
-        self.expect('Infix')
+        self.expect("Infix")
         expr3 = self.parse_exp(q + 1)
         return Node(expr2, expr1, expr3)
 
     def e_Postfix(self, expr1, token, p):
-        q = left_binary_ops['Postfix']
+        q = left_binary_ops["Postfix"]
         if q < p:
             return None
         self.consume()
@@ -579,32 +608,38 @@ class Parser(object):
         return Node(expr1, expr2)
 
     def e_ApplyList(self, expr1, token, p):
-        q = right_binary_ops['Apply']
+        q = right_binary_ops["Apply"]
         if q < p:
             return None
         self.consume()
         expr2 = self.parse_exp(q)
-        expr3 = Node('List', Number('1'))
-        return Node('Apply', expr1, expr2, expr3)
+        expr3 = Node("List", Number("1"))
+        return Node("Apply", expr1, expr2, expr3)
 
     def e_Function(self, expr1, token, p):
-        q = postfix_ops['Function']
+        q = postfix_ops["Function"]
         if q < p:
             return None
         # postfix or right-binary determined by symbol
         self.consume()
-        if token.text == '&':
-            return Node('Function', expr1)
+        if token.text == "&":
+            return Node("Function", expr1)
         else:
             expr2 = self.parse_exp(q)
-            return Node('Function', expr1, expr2)
+            return Node("Function", expr1, expr2)
 
     def e_RawColon(self, expr1, token, p):
         head_name = expr1.get_head_name()
-        if head_name == 'Symbol':
-            head = 'Pattern'
-        elif head_name in ('Blank', 'BlankSequence', 'BlankNullSequence', 'Pattern', 'Optional'):
-            head = 'Optional'
+        if head_name == "Symbol":
+            head = "Pattern"
+        elif head_name in (
+            "Blank",
+            "BlankSequence",
+            "BlankNullSequence",
+            "Pattern",
+            "Optional",
+        ):
+            head = "Optional"
         else:
             return None
         q = all_ops[head]
@@ -615,7 +650,7 @@ class Parser(object):
         return Node(head, expr1, expr2)
 
     def e_Semicolon(self, expr1, token, p):
-        q = flat_binary_ops['CompoundExpression']
+        q = flat_binary_ops["CompoundExpression"]
         if q < p:
             return None
         self.consume()
@@ -626,9 +661,9 @@ class Parser(object):
 
         # So that e.g. 'x = 1;' doesn't wait for newline in the frontend
         tag = self.next().tag
-        if tag == 'END' and self.bracket_depth == 0:
-            expr2 = Symbol('Null')
-            return Node('CompoundExpression', expr1, expr2).flatten()
+        if tag == "END" and self.bracket_depth == 0:
+            expr2 = Symbol("Null")
+            return Node("CompoundExpression", expr1, expr2).flatten()
 
         # XXX look for next expr otherwise backtrack
         try:
@@ -636,23 +671,23 @@ class Parser(object):
         except TranslateError:
             self.backtrack(pos)
             self.feeder.messages = messages
-            expr2 = Symbol('Null')
-        return Node('CompoundExpression', expr1, expr2).flatten()
+            expr2 = Symbol("Null")
+        return Node("CompoundExpression", expr1, expr2).flatten()
 
     def e_Minus(self, expr1, token, p):
-        q = left_binary_ops['Subtract']
+        q = left_binary_ops["Subtract"]
         if q < p:
             return None
         self.consume()
         expr2 = self.parse_exp(q + 1)
-        if isinstance(expr2, Number) and not expr2.value.startswith('-'):
-            expr2.value = '-' + expr2.value
+        if isinstance(expr2, Number) and not expr2.value.startswith("-"):
+            expr2.value = "-" + expr2.value
         else:
-            expr2 = Node('Times', Number('1', sign=-1), expr2).flatten()
-        return Node('Plus', expr1, expr2).flatten()
+            expr2 = Node("Times", Number("1", sign=-1), expr2).flatten()
+        return Node("Plus", expr1, expr2).flatten()
 
     def e_TagSet(self, expr1, token, p):
-        q = all_ops['Set']
+        q = all_ops["Set"]
         if q < p:
             return None
         self.consume()
@@ -660,71 +695,73 @@ class Parser(object):
         # examine next token
         token = self.next_noend()
         tag = token.tag
-        if tag == 'Set':
-            head = 'TagSet'
-        elif tag == 'SetDelayed':
-            head = 'TagSetDelayed'
-        elif tag == 'Unset':
-            head = 'TagUnset'
+        if tag == "Set":
+            head = "TagSet"
+        elif tag == "SetDelayed":
+            head = "TagSetDelayed"
+        elif tag == "Unset":
+            head = "TagUnset"
         else:
             self.tokeniser.sntx_message(token.pos)
             raise InvalidSyntaxError()
         self.consume()
-        if head == 'TagUnset':
+        if head == "TagUnset":
             return Node(head, expr1, expr2)
         expr3 = self.parse_exp(q + 1)
         return Node(head, expr1, expr2, expr3)
 
     def e_Unset(self, expr1, token, p):
-        q = all_ops['Set']
+        q = all_ops["Set"]
         if q < p:
             return None
         self.consume()
-        return Node('Unset', expr1)
+        return Node("Unset", expr1)
 
     def e_Derivative(self, expr1, token, p):
-        q = postfix_ops['Derivative']
+        q = postfix_ops["Derivative"]
         if q < p:
             return None
         n = 0
-        while self.next().tag == 'Derivative':
+        while self.next().tag == "Derivative":
             self.consume()
             n += 1
-        head = Node('Derivative', Number(str(n)))
+        head = Node("Derivative", Number(str(n)))
         return Node(head, expr1)
 
     def e_Divide(self, expr1, token, p):
-        q = left_binary_ops['Divide']
+        q = left_binary_ops["Divide"]
         if q < p:
             return None
         self.consume()
         expr2 = self.parse_exp(q + 1)
-        return Node('Times', expr1, Node('Power', expr2, Number('1', sign=-1))).flatten()
+        return Node(
+            "Times", expr1, Node("Power", expr2, Number("1", sign=-1))
+        ).flatten()
 
     def e_Alternatives(self, expr1, token, p):
-        q = flat_binary_ops['Alternatives']
+        q = flat_binary_ops["Alternatives"]
         if q < p:
             return None
         self.consume()
         expr2 = self.parse_exp(q + 1)
-        return Node('Alternatives', expr1, expr2).flatten()
+        return Node("Alternatives", expr1, expr2).flatten()
 
     def e_MessageName(self, expr1, token, p):
         leaves = [expr1]
-        while self.next().tag == 'MessageName':
+        while self.next().tag == "MessageName":
             self.consume()
             token = self.next()
-            if token.tag == 'Symbol':
+            if token.tag == "Symbol":
                 # silently convert Symbol to String
                 self.consume()
                 leaf = String(token.text)
-            elif token.tag == 'String':
+            elif token.tag == "String":
                 leaf = self.p_String(token)
             else:
                 self.tokeniser.sntx_message(token.pos)
                 raise InvalidSyntaxError()
             leaves.append(leaf)
-        return Node('MessageName', *leaves)
+        return Node("MessageName", *leaves)
 
     # B methods
     #
@@ -737,95 +774,95 @@ class Parser(object):
         if box0 is not None:
             return None
         self.consume()
-        q = misc_ops['SqrtBox']
+        q = misc_ops["SqrtBox"]
         box1 = self.parse_box(q)
-        if self.next().tag == 'OtherscriptBox':
+        if self.next().tag == "OtherscriptBox":
             self.consume()
             box2 = self.parse_box(q)
-            return Node('RadicalBox', box1, box2)
+            return Node("RadicalBox", box1, box2)
         else:
-            return Node('SqrtBox', box1)
+            return Node("SqrtBox", box1)
 
     def b_SuperscriptBox(self, box1, token, p):
-        q = misc_ops['SuperscriptBox']
+        q = misc_ops["SuperscriptBox"]
         if q < p:
             return None
         if box1 is None:
-            box1 = String('')
+            box1 = String("")
         self.consume()
         box2 = self.parse_box(q)
-        if self.next().tag == 'OtherscriptBox':
+        if self.next().tag == "OtherscriptBox":
             self.consume()
-            box3 = self.parse_box(misc_ops['SubsuperscriptBox'])
-            return Node('SubsuperscriptBox', box1, box3, box2)
+            box3 = self.parse_box(misc_ops["SubsuperscriptBox"])
+            return Node("SubsuperscriptBox", box1, box3, box2)
         else:
-            return Node('SuperscriptBox', box1, box2)
+            return Node("SuperscriptBox", box1, box2)
 
     def b_SubscriptBox(self, box1, token, p):
-        q = misc_ops['SubscriptBox']
+        q = misc_ops["SubscriptBox"]
         if q < p:
             return None
         if box1 is None:
-            box1 = String('')
+            box1 = String("")
         self.consume()
         box2 = self.parse_box(q)
-        if self.next().tag == 'OtherscriptBox':
+        if self.next().tag == "OtherscriptBox":
             self.consume()
-            box3 = self.parse_box(misc_ops['SubsuperscriptBox'])
-            return Node('SubsuperscriptBox', box1, box2, box3)
+            box3 = self.parse_box(misc_ops["SubsuperscriptBox"])
+            return Node("SubsuperscriptBox", box1, box2, box3)
         else:
-            return Node('SubscriptBox', box1, box2)
+            return Node("SubscriptBox", box1, box2)
 
     def b_UnderscriptBox(self, box1, token, p):
-        q = misc_ops['UnderscriptBox']
+        q = misc_ops["UnderscriptBox"]
         if q < p:
             return None
         if box1 is None:
-            box1 = String('')
+            box1 = String("")
         self.consume()
         box2 = self.parse_box(q)
-        if self.next().tag == 'OtherscriptBox':
+        if self.next().tag == "OtherscriptBox":
             self.consume()
-            box3 = self.parse_box(misc_ops['UnderoverscriptBox'])
-            return Node('UnderoverscriptBox', box1, box2, box3)
+            box3 = self.parse_box(misc_ops["UnderoverscriptBox"])
+            return Node("UnderoverscriptBox", box1, box2, box3)
         else:
-            return Node('UnderscriptBox', box1, box2)
+            return Node("UnderscriptBox", box1, box2)
 
     def b_FractionBox(self, box1, token, p):
-        q = misc_ops['FractionBox']
+        q = misc_ops["FractionBox"]
         if q < p:
             return None
         if box1 is None:
-            box1 = String('')
+            box1 = String("")
         self.consume()
         box2 = self.parse_box(q + 1)
-        return Node('FractionBox', box1, box2)
+        return Node("FractionBox", box1, box2)
 
     def b_FormBox(self, box1, token, p):
-        q = misc_ops['FormBox']
+        q = misc_ops["FormBox"]
         if q < p:
             return None
         if box1 is None:
-            box1 = Symbol('StandardForm')   # RawForm
+            box1 = Symbol("StandardForm")  # RawForm
         elif is_symbol_name(box1.value):
             box1 = Symbol(box1.value, context=None)
         else:
-            box1 = Node('Removed', String('$$Failure'))
+            box1 = Node("Removed", String("$$Failure"))
         self.consume()
         box2 = self.parse_box(q)
-        return Node('FormBox', box2, box1)
+        return Node("FormBox", box2, box1)
 
     def b_OverscriptBox(self, box1, token, p):
-        q = misc_ops['OverscriptBox']
+        q = misc_ops["OverscriptBox"]
         if q < p:
             return None
         if box1 is None:
-            box1 = String('')
+            box1 = String("")
         self.consume()
         box2 = self.parse_box(q)
-        if self.next().tag == 'OtherscriptBox':
+        if self.next().tag == "OtherscriptBox":
             self.consume()
-            box3 = self.parse_box(misc_ops['UnderoverscriptBox'])
-            return Node('UnderoverscriptBox', box1, box3, box2)
+            box3 = self.parse_box(misc_ops["UnderoverscriptBox"])
+            return Node("UnderoverscriptBox", box1, box3, box2)
         else:
-            return Node('OverscriptBox', box1, box2)
+            return Node("OverscriptBox", box1, box2)
