@@ -9,9 +9,10 @@ session = MathicsSession(add_builtin=True, catch_interrupt=False)
 def check_evaluation(str_expr: str, str_expected: str, message=""):
     """Helper function to test that a WL expression against
     its results"""
-    result = session.evaluate(str_expr)
-    expected = session.evaluate(str_expected)
-
+    result = session.evaluate("ToString[" + str_expr + "]").value
+    print("result=",result)
+    expected = session.evaluate("ToString[" + str_expected + "]").value
+    print("expected=",expected)
     print(time.asctime())
     print(message)
     if message:
